/**
 * Ephoto Dam Addon for Drupal 9
 *
 * @base EPHOTO DAM FIELD
 * @author EINDEN
 */

(function ($) {
  Drupal.ephotoDamField = {
    /**
     * EPHOTO ID of Drupal Addon
     */		
    CLIENT_ID: 'CaCvqS5H',
    
    /**
     * Settings
     */    
    settings: drupalSettings.ephotoDamField,

    /**
     * Current authid
     */     
    authid: null,

    /**
     * Ephoto Dam instance
     */		
    ephoto: null,
    
    /**
     * Ephoto Dam Server connected
     */		
    connected: false, 
    
    /**
     * Current selected field
     */
    fieldSelected : null,

    /**
     * Enable version support
     */    
    enableVersionSupport: function(el) {
      if (el.checked) {
        $('.ephoto-dam-field-version-field').removeClass('ephoto-dam-field-hidden');
        $('.ephoto-dam-field-version-prefix').removeClass('ephoto-dam-field-hidden');
        
        $('.ephoto-dam-field-versions-button').removeClass('ephoto-dam-field-hidden');
        $('.ephoto-dam-field-file-button').addClass('ephoto-dam-field-hidden');
      }

      else {
        $('.ephoto-dam-field-version-field').addClass('ephoto-dam-field-hidden');
        $('.ephoto-dam-field-version-prefix').addClass('ephoto-dam-field-hidden');        
        
        $('.ephoto-dam-field-versions-button').addClass('ephoto-dam-field-hidden');
        $('.ephoto-dam-field-file-button').removeClass('ephoto-dam-field-hidden');
      }
    },

    /**
     * Load Ephoto Dam Api
     */
    load: function() {
        if (this.settings && this.settings.server_url) {
            $.getScript(this.settings.server_url + '/api/apiJS.js', this.init.bind(this));
        }
    },
    
    /**
     * Init Ephoto Dam Api
     */    
    init: function() {
        var self = this;

        if (!ePhoto) {
            console.log('Error : loading the Ephoto Dam API');
            return;
        }

        this.authid = $.cookie('drupal_ephoto_dam_authid');

        this.ephoto = new ePhoto({
            'server': this.settings.server_url,
            'authID': this.authid,
            'client': this.CLIENT_ID
        });

        this.ephoto.connect();

        this.ephoto.File.enableDCore('json');

        this.ephoto.callOnConnect(function() {
            let version = self.ephoto.getVersion().split('.');

            // Requires API version 6.0 (Ephoto Dam 4.0)
            if (version.length !==3 || version[0] < 6) {
                alert(Drupal.t('The version of Ephoto Dam is not compatible with this feature'));
                return;
            }

            self.authid = self.ephoto.getAuthID();

            $.cookie('drupal_ephoto_dam_authid', self.authid, { path:'/' });

            self.connected = true;
        });
    },

    /**
     * Select a file versions
     */
    selectVersions: function() {
        if (!this._getFields()) {
            console.log('Error : field selection error');
            return;
        }

        this.ephoto.File.callOnFileReceived(this.getVersionsUrls.bind(this));

        this.ephoto.File.setButtons( this.ephoto.IMAGE_FILES, [ {
            'mode': 'none'
        } ] );  

        this.ephoto.File.get();
    },

    /**
     * Get URL of selected file versions
     *
     * @param {null} None value
     * @param {object} DublinCore values
     */    
    getVersionsUrls: function(empty, dcore) {
        if (dcore === 'fileDoesNotExist') {
            console.log('Notice : file does not exist');
            return;
        }

        const id = parseInt(dcore['dc:identifier:number'], 10);
        const constid = dcore['dc:identifier:string'];
        const thumbnail = dcore['dc:description.thumbnail'];

        if (!id || !constid || !thumbnail) {
            console.log('Notice : dublincore values missing');
            return;
        }

        const caption = this._buildCaption(dcore);        

        this.ephoto.File.getVersions(id, function(versions) {
            const fields = this._getFields();

            for (const inc in fields) {
                if (typeof versions[fields[inc].name] === 'undefined') {
                    fields[inc].input.val('');
                    fields[inc].caption.val('');
                    fields[inc].thumbnail.val('');

                    fields[inc].thumbnail.prev().attr('src', '');
                    continue;
                }

                let size = this._checkSizeValue(fields[inc].size);
                let version = fields[inc].name;
                let file = version + '@' + id;

                let button = this.ephoto.File.setButtons( this.ephoto.IMAGE_FILES, [ {
                    'mode': 'link',
                    'definition': 'custom',
                    'size': size

                } ] ).get(0);

                button.getLink([file], function(url) {
                    fields[inc].input.val(url);
                    fields[inc].caption.val(caption);
                    fields[inc].thumbnail.val(thumbnail);

                    fields[inc].thumbnail.prev().attr('src', thumbnail);
                }); 
            }

        }.bind(this));
    },

    /**
     * Select a file
     * 
     * @param {number} Field index
     */    
    selectFile: function(delta) {
        this.fieldSelected = this._getFields()[delta];

        if (!this.fieldSelected) {
            console.log('Error : field selection error');
            return;
        }

        let size = this._checkSizeValue(this.fieldSelected.size);

        if (size === '' || size === 0) return;

        this.ephoto.File.callOnFileReceived(this.getFileUrl.bind(this));

        this.ephoto.File.setButtons( this.ephoto.IMAGE_FILES, [ {
            'mode': 'link',
            'definition': 'custom',
            'size': size
        } ] );        

        this.ephoto.File.get();
    },

    /**
     * Get URL of selected file
     * 
     * @param {string} URL value
     * @param {object} DublinCore values
     */    
    getFileUrl: function(url, dcore) {
        if (url === 'fileDoesNotExist') {
            console.log('Notice : file does not exist');
            return;
        }

        if (!this.fieldSelected) {
            console.log('Notice : field selected missing');
            return;
        }

        const thumbnail = dcore['dc:description.thumbnail'];
        const caption = this._buildCaption(dcore); 

        this.fieldSelected.input.val(url);
        this.fieldSelected.caption.val(caption);
        this.fieldSelected.thumbnail.val(thumbnail);
        
        this.fieldSelected.thumbnail.prev().attr('src', thumbnail);
    },

    /**
     * Check size value
     * 
     * @param {number} A size value
     */    
    _checkSizeValue: function(size) {
        return size.toString().split('x').map(function(val) {
            val = val === '' || isNaN(val) ? 0 : parseInt(val, 10);
            return (val > 1600) ? 1600 : ((val <= 0) ? '' : val);

        }).join('x');
    },

    /**
     * Get all fields
     */
    _getFields: function() {
        var fields = [];
        var sizes = $('.ephoto-dam-field-imagesize-field').toArray();
        var inputs = $('.ephoto-dam-field-url-field').toArray();
        var captions = $('.ephoto-dam-field-caption-field').toArray();
        var thumbnails = $('.ephoto-dam-field-thumbnail').toArray();

        $('.ephoto-dam-field-version-field').map(function(index, el) {
            fields.push({
                'name' : $(el).val(),
                'size' : $(sizes[index]).val(),
                'input' : $(inputs[index]),
                'caption' : $(captions[index]),
                'thumbnail' : $(thumbnails[index])
            });
        });

        return fields;
    },
    
    /**
     * Build the caption width a DublinCore
     */		
    _buildCaption: function(dcore) {
        var result = this.settings.captions_format;

        for (const property in dcore) {
            result = result.replace('[' + property + ']', dcore[property]);
        }

        return result;
    },
  }

})(jQuery);

Drupal.ephotoDamField.load();
